unit SteIconDBGrid;

        //===========================================================//
        //  Componente TSteIconDBGrid - (C) Stefano Barbieri - 2004  //
        //  v 1.0                                                    //
        //                                                           //
        //  Consente di visualizzare i valori boolean dei campi di   //
        //  una DBGrid con una immagine bitmap associata, contenuta  //
        //  in una ImageList.                                        //
        //                                                           //
        //      Per segnalare errori o suggerire miglioramenti:      //
        //                 ste_barbieri@virgilio.it                  //
        //                                                           //
        //===========================================================//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, Db, DBGrids;

type
  TSteIconDBGrid = class(TDBGrid)
  private
    FAutoCheck: boolean;
    FCanStoreColumnsWidth: boolean;
    FImages: TImageList;
    FImageIndexes: array[0..255] of ShortInt;
    FImageHeight: Integer;
    FImageWidth: Integer;
    FIniFileName: string;
    FWidthLoaded: boolean;
    FShowImages: boolean;

    FOnCellClick: TNotifyEvent;
    FOnColumnMoved: TMovedEvent;

    procedure SetImages(const Value: TImageList);
    procedure SetImageWidth(const Value: Integer);
    procedure SetShowImages(const Value: boolean);
    function  GetImgIndexes(Index: Integer): ShortInt;
    procedure SetImgIndexes(Index: Integer; const Value: ShortInt);
    procedure SetAutoCheck(const Value: boolean);
    procedure SetIniFileName(const Value: string);
    procedure SetCanStoreColumnsWidth(const Value: boolean);
    procedure SetImageHeight(const Value: Integer);

  protected
    procedure DrawColumnCell(const Rect: TRect; DataCol: Integer;
                             Column: TColumn; State: TGridDrawState); override;
    procedure CellClick(Column: TColumn); override;
    procedure ColumnMoved(FromIndex, ToIndex: Longint); override;
  public
    constructor Create(aOwner: TComponent); override;
    property  ImageIndexes[Index: Integer]: ShortInt read GetImgIndexes write SetImgIndexes;
    function  LoadColumnsWidth: boolean;
    procedure Notification(aComponent: TComponent;
                           aOperation: TOperation); override;
    function  StoreColumnsWidth: boolean;
  published
    property AutoCheck: boolean read FAutoCheck write SetAutoCheck default false;
    property CanStoreColumnsWidth: boolean read FCanStoreColumnsWidth write SetCanStoreColumnsWidth default false;
    property Images: TImageList read FImages write SetImages default nil;
    property ImageHeight: Integer read FImageHeight write SetImageHeight default 16;
    property ImageWidth: Integer read FImageWidth write SetImageWidth default 16;
    property IniFileName: string read FIniFileName write SetIniFileName;
    property ShowImages: boolean read FShowImages write SetShowImages default true;
    property OnCellClick: TNotifyEvent read FOnCellClick write FOnCellClick;
    property OnColumnMoved: TMovedEvent read FOnColumnMoved write FOnColumnMoved;
  end;

procedure Register;

implementation

 uses IniFiles;

 const ERROR_STR = 'Errore nell''accesso al file ';

procedure Register;
begin
  RegisterComponents('SteDataControls', [TSteIconDBGrid]);
end;

{ TSteIconDBGrid }

procedure TSteIconDBGrid.CellClick(Column: TColumn);
begin
 if Assigned(FOnCellClick) then FOnCellClick(Self);

 if (Column.Field is TBooleanField) and (FAutoCheck)
 then
  begin
   Column.Field.DataSet.Edit;
   Column.Field.Value:= not (Column.Field.Value);
   Column.Field.DataSet.Post;
  end;

  Invalidate;
end;

procedure TSteIconDBGrid.ColumnMoved(FromIndex, ToIndex: Longint);
var
  Container: Integer;
  i: Integer;
begin

 if FromIndex <> ToIndex
  then
   begin
    // adegua l'index all'array 0-based ImageIndexes
    i:= FromIndex - 1;

    // Conserva l'ImageIndex di posto originario
    Container:= ImageIndexes[i];

    // Shifta tutti gli ImageIndexes compresi tra l'origine e la destinazione
    // se lo spostamento avviene da dx verso sx
    if FromIndex > ToIndex
    then
     begin
      while not (i < ToIndex) do
       begin
        ImageIndexes[i]:= ImageIndexes[i-1];
        dec(i);
       end;
     end
    // se lo spostamento avviene da sx verso dx
    else { if FormIndex < ToIndex }
     begin
      while not (i > (ToIndex - 1)) do
       begin
        ImageIndexes[i]:= ImageIndexes[i + 1];
        inc(i);
       end;
     end;

    // Riassegna alla colonna origine cambiata di posto il suo ImageIndex
    ImageIndexes[ToIndex - 1]:= Container;
   end;

 inherited;
end;

constructor TSteIconDBGrid.Create(aOwner: TComponent);
var
  i: Integer;
begin

 inherited Create(aOwner);

 // inizializza la propriet AutoCheck
 FAutoCheck:= false;

 // inizializza la propriet CanStoreColumnsWidth
 FCanStoreColumnsWidth := false;

 // inizializza la property ImageList;
 FImages:= nil;

 // inizializza tutti gli ImageIndexes
 i:= 0;
 repeat
  FImageIndexes[i]:= -1;
  inc(i);
 until i > 255;

 // inizializza le property ImageWidth e ImageHeight
 FImageHeight:= 16;
 FImageWidth:= 16;

 // inizializza la propriet IniFileName
 FIniFileName:= '';

 // inizializza la property ShowImages
 FShowImages:= true;

 FWidthLoaded:= false;

end;

procedure TSteIconDBGrid.DrawColumnCell(const Rect: TRect;
  DataCol: Integer; Column: TColumn; State: TGridDrawState);
var
  Bmp: TBitmap;
  BmpRect: TRect;
  RectHeight,
  RectWidth: Integer;
begin
  if (Column.Field is TBooleanField) and
     (FShowImages) and
     (Assigned(FImages)) and
     (FImageIndexes[Column.Index] <> -1)

  then
   begin
    // pulisce l'esistente
    Canvas.FillRect (Rect);

    // prosegue solo se il Field.Value  true;
    if (Column.Field.Value)
    then
     begin
      // copia il Rect
      BmpRect := Rect;

      // Calcola altezza e larghezza della cella
      RectHeight:= Rect.Bottom - Rect.Top;
      RectWidth:= Rect.Right - Rect.Left;

      // Restinge BmpRect alle dimensioni delle immagini impostate
      InflateRect (BmpRect, - ( RectWidth - FImageWidth),
                            -(RectHeight - FImageHeight));

      // Centra l'immagine nella cella
      BmpRect.TopLeft.X:= Rect.TopLeft.X + ((RectWidth - FImageWidth) div 2);
      BmpRect.TopLeft.Y:= Rect.TopLeft.Y + ((RectHeight - FImageHeight) div 2);

      Bmp := TBitmap.Create;
      try
       FImages.GetBitmap(FImageIndexes[Column.Index] , Bmp);
       Canvas.Draw(BmpRect.TopLeft.X, BmpRect.TopLeft.Y, Bmp);
      finally
       Bmp.Free;
      end;
     end;
   end

  else
   // se la cella non  di tipo TBooleanField
   inherited DrawColumnCell(Rect, DataCol, Column, State);

end;

function TSteIconDBGrid.GetImgIndexes(Index: Integer): ShortInt;
begin
 Result:= FImageIndexes[Index];
end;

function TSteIconDBGrid.LoadColumnsWidth: boolean;
var
  IniFile: TIniFile;
  i: Integer;
begin
 if (not FWidthLoaded) and (FIniFileName <> '') and (FileExists(FIniFileName))
 then
  begin
  try
   try
    IniFile:= TIniFile.Create(FIniFileName);

    i:= 0;
    repeat
     Columns[i].Width:= IniFile.ReadInteger(UpperCase(Self.Name),
                                            Columns[i].Title.Caption,
                                            100);
     inc(i);
    until i = Columns.Count;

    Result:= True;
    FWidthLoaded:= True;
   except
    MessageDlg(ERROR_STR + FIniFileName, mtError, [mbOK], 0);
    Result:= False;
   end;
  finally
   IniFile.Free;
  end;
 end
 else
  Result:= False;
end;

procedure TSteIconDBGrid.Notification(aComponent: TComponent;
  aOperation: TOperation);
begin
  inherited;
  if (aComponent = FImages) and (aOperation = opRemove)
  then
   FImages:= nil;
end;

procedure TSteIconDBGrid.SetAutoCheck(const Value: boolean);
begin
 if FAutoCheck <> Value
 then
  FAutoCheck := Value;
end;

procedure TSteIconDBGrid.SetCanStoreColumnsWidth(const Value: boolean);
begin
 if FCanStoreColumnsWidth <> Value
 then
  FCanStoreColumnsWidth := Value;
 if not FCanStoreColumnsWidth
 then
  try
   DeleteFile(FIniFileName);
  except
   MessageDlg(ERROR_STR + FIniFileName, mtError, [mbOK], 0);
  end;
end;

procedure TSteIconDBGrid.SetImageHeight(const Value: Integer);
begin
  if FImageHeight <> Value
 then
  begin
   FImageHeight := Value;
   Invalidate;
  end;
end;

procedure TSteIconDBGrid.SetImages(const Value: TImageList);
begin
if FImages <> Value
 then
  begin
   FImages:= Value;
   if FImages <> nil
   then
    // SteIconDBGrid aggiunge se stessa alla lista dei componenti da "avvisare"
    // quando la ImageList viene distrutta in modo che venga correttamente
    // chiamato il metodo Notification
    FImages.FreeNotification(Self);

   // Fa s che la Grid venga correttamente ridisegnata se cambia il "Set" delle
   // immagini o se la ImageList viene eliminata
   Invalidate;
  end;
end;

procedure TSteIconDBGrid.SetImageWidth(const Value: Integer);
begin
 if FImageWidth <> Value
 then
  begin
   FImageWidth := Value;
   Invalidate;
  end;
end;

procedure TSteIconDBGrid.SetImgIndexes(Index: Integer;
  const Value: ShortInt);
begin
 FImageIndexes[Index]:= Value;
end;

procedure TSteIconDBGrid.SetIniFileName(const Value: string);
begin
 if FIniFileName <> Value
 then
  FIniFileName := Value;
end;

procedure TSteIconDBGrid.SetShowImages(const Value: boolean);
begin
 if ShowImages <> Value
 then
  begin
   FShowImages := Value;
   Invalidate;
  end;
end;

function TSteIconDBGrid.StoreColumnsWidth: boolean;
var
  IniFile: TIniFile;
  i: Integer;
begin
 if (FCanStoreColumnsWidth) and (FIniFileName <> '')
 then
  begin
  try
   try
    IniFile:= TIniFile.Create(FIniFileName);

    i:= 0;
    repeat
     IniFile.WriteInteger(UpperCase(Self.Name),
                          Columns[i].Title.Caption,
                          Columns[i].Width);
     inc(i);
    until i = Columns.Count;

    Result:= True;
   except
    MessageDlg(ERROR_STR + FIniFileName, mtError, [mbOK], 0);
    Result:= False;
   end;
  finally
   IniFile.Free;
  end;
 end
 else
  Result:= False;
end;
end.
